@@ -4,9 +4,10 @@
  *
  *   Purpose:	Implements class interface for netCDF over C interface
  *
- *   $Header: /upc/share/CVS/netcdf-3/cxx/netcdf.cpp,v 1.4 2001/02/05 20:29:12 russ Exp $
+ *   $Header: /upc/share/CVS/netcdf-3/cxx/netcdf.cpp,v 1.11 2005/11/05 01:39:52 ed Exp $
  *********************************************************************/
 
+#include <config.h>
 #include <string.h>
 #include <stdlib.h>
 #include "netcdfcpp.h"
@@ -218,6 +219,24 @@ NcFile::FillMode NcFile::get_fill( void ) const
     return the_fill_mode;
 }
 
+NcFile::FileFormat NcFile::get_format( void ) const
+{
+    int the_format;
+    nc_inq_format(the_id, &the_format);
+    switch (the_format) {
+    case NC_FORMAT_CLASSIC:
+	return Classic;
+    case NC_FORMAT_64BIT:
+	return Offset64Bits;
+    case NC_FORMAT_NETCDF4:
+	return Netcdf4;
+    case NC_FORMAT_NETCDF4_CLASSIC:
+	return Netcdf4Classic;
+    default:
+	return BadFormat;
+    }
+}
+
 NcBool NcFile::sync( void )
 {
     if (!data_mode())
@@ -295,13 +314,23 @@ int NcFile::id( void ) const
 }
 
 NcFile::NcFile( const char* path, FileMode fmode, 
-		size_t* chunksizeptr, size_t initialsize  )
+		size_t* chunksizeptr, size_t initialsize, FileFormat fformat  )
 {
     NcError err(NcError::silent_nonfatal); // constructor must not fail
 
     int mode = NC_NOWRITE;
     the_fill_mode = Fill;
     int status;
+    
+    // If the user wants a 64-bit offset format, set that flag.
+    if (fformat == Offset64Bits)
+       mode |= NC_64BIT_OFFSET;
+#ifdef USE_NETCDF4
+    else if (fformat == Netcdf4)
+       mode |= NC_NETCDF4;
+    else if (fformat == Netcdf4Classic)
+       mode |= NC_NETCDF4|NC_CLASSIC_MODEL;
+#endif
 
     switch (fmode) {
     case Write:
@@ -505,6 +534,7 @@ NcValues* NcTypedComponent::get_space( long numVals ) const
 NcVar::~NcVar( void )
 {
     delete[] the_cur;
+    delete[] cur_rec;
     delete[] the_name;
 }
 
@@ -605,23 +635,52 @@ NcValues* NcVar::values( void ) const
     return valp;
 }
 
+int NcVar::dim_to_index(NcDim *rdim)
+{
+  for (int i=0; i < num_dims() ; i++) {
+    if (strcmp(get_dim(i)->name(),rdim->name()) == 0) {
+      return i;
+    }
+  }
+  // we should fail and gripe about it here....
+  return -1;
+}
+
+void NcVar::set_rec(NcDim *rdim, long slice)
+{
+  int i = dim_to_index(rdim);
+  // we should fail and gripe about it here....
+  if (slice >= get_dim(i)->size() && ! get_dim(i)->is_unlimited())
+	  return;  
+  cur_rec[i] = slice;
+  return;
+} 
+
 void NcVar::set_rec(long rec)
 {
-  cur_rec = rec;
+  // Since we can't ask for the record dimension here
+  // just assume [0] is it.....
+  set_rec(get_dim(0),rec);
   return;
 } 
 
 NcValues* NcVar::get_rec(void)
 {
-    return get_rec(cur_rec);
+    return get_rec(get_dim(0), cur_rec[0]);
 }
 
 NcValues* NcVar::get_rec(long rec)
 {
+    return get_rec(get_dim(0), rec);
+}
+
+NcValues* NcVar::get_rec(NcDim* rdim, long slice)
+{
+    int idx = dim_to_index(rdim);
     long size = num_dims();
     long* start = new long[size];
     for (int i=1; i < size ; i++) start[i] = 0;
-    start[0] = rec;
+    start[idx] = slice;
     NcBool result = set_cur(start);
     if (! result ) {
 	delete [] start;
@@ -629,8 +688,8 @@ NcValues* NcVar::get_rec(long rec)
     }
 
     long* edge = edges();
-    edge[0] = 1;
-    NcValues* valp = get_space(rec_size());
+    edge[idx] = 1;
+    NcValues* valp = get_space(rec_size(rdim));
     if (ncvarget(the_file->id(), the_id, start, edge, valp->base()) == ncBad) {
 	delete [] start;
 	delete [] edge;
@@ -646,23 +705,36 @@ NcValues* NcVar::get_rec(long rec)
 #define NcVar_put_rec(TYPE)                                                   \
 NcBool NcVar::put_rec( const TYPE* vals)                                      \
 {                                                                             \
-    return put_rec(vals, cur_rec);                                            \
+    return put_rec(get_dim(0), vals, cur_rec[0]);                             \
+}                                                                             \
+                                                                              \
+NcBool NcVar::put_rec( NcDim *rdim, const TYPE* vals)                         \
+{                                                                             \
+    int idx = dim_to_index(rdim);                                             \
+    return put_rec(rdim, vals, cur_rec[idx]);                                 \
 }                                                                             \
                                                                               \
 NcBool NcVar::put_rec( const TYPE* vals,                                      \
                      long rec)                                                \
 {                                                                             \
+   return put_rec(get_dim(0), vals, rec);                                     \
+}                                                                             \
+                                                                              \
+NcBool NcVar::put_rec( NcDim* rdim, const TYPE* vals,                         \
+                     long slice)                                              \
+{                                                                             \
+    int idx = dim_to_index(rdim);                                             \
     long size = num_dims();                                                   \
     long* start = new long[size];                                             \
     for (int i=1; i < size ; i++) start[i] = 0;                               \
-    start[0] = rec;                                                           \
+    start[idx] = slice;                                                       \
     NcBool result = set_cur(start);                                           \
     delete [] start;                                                          \
     if (! result )                                                            \
       return FALSE;                                                           \
                                                                               \
     long* edge = edges();                                                     \
-    edge[0] = 1;                                                              \
+    edge[idx] = 1;                                                            \
     result = put(vals, edge);                                                 \
     delete [] edge;                                                           \
     return result;                                                            \
@@ -677,27 +749,41 @@ NcVar_put_rec(float)
 NcVar_put_rec(double)
 
 long NcVar::rec_size(void) {
+    return rec_size(get_dim(0));
+}
+
+long NcVar::rec_size(NcDim *rdim) {
+    int idx = dim_to_index(rdim); 
     long size = 1;
     long* edge = edges();
-    for( int i = num_dims()-1 ; i>0 ; i--)
-	size *= edge[i];
+    for( int i = 0 ; i<num_dims() ; i++) {
+	if (i != idx) {
+	  size *= edge[i];
+	}
+    }
     delete [] edge;
     return size;
 }
 
 #define NcVar_get_index(TYPE)                                                 \
 long NcVar::get_index(const TYPE* key)                                        \
+{                                                                             \
+   return get_index(get_dim(0), key);                                         \
+}                                                                             \
+                                                                              \
+long NcVar::get_index(NcDim *rdim, const TYPE* key)                           \
 {                                                                             \
 if (type() != NcTypeEnum(TYPE))                                               \
     return -1;                                                                \
 if (! the_file->data_mode())                                                  \
     return -1;                                                                \
-long maxrec = get_dim(0)->size();                                             \
-long maxvals = rec_size();                                                    \
+int idx = dim_to_index(rdim);                                                 \
+long maxrec = get_dim(idx)->size();                                           \
+long maxvals = rec_size(rdim);                                                \
 NcValues* val;                                                                \
-int validx;                                                                  \
+int validx;                                                                   \
 for (long j=0; j<maxrec; j++) {                                               \
-    val = get_rec(j);                                                         \
+    val = get_rec(rdim,j);                                                    \
     if (val == NULL) return -1;                                               \
     for (validx = 0; validx < maxvals; validx++) {                            \
         if (key[validx] != val->as_ ## TYPE(validx)) break;                   \
@@ -743,7 +829,7 @@ NcBool NcVar::put( const TYPE* vals,					      \
 	} else								      \
 	  break;							      \
     }									      \
-    static long start[5] = {0, 0, 0, 0, 0};				      \
+    long start[5];                      				      \
     for (int j = 0; j < 5; j++) {					      \
      start[j] = the_cur[j];						      \
     }									      \
@@ -964,6 +1050,9 @@ NcBool NcVar::sync(void)
     if (the_cur) {
 	delete [] the_cur;
     }
+    if (cur_rec) {
+	delete [] cur_rec;
+    }
     char nam[MAX_NC_NAME];
     if (the_file 
 	&& ncvarinq(the_file->id(), the_id, nam, 0, 0, 0, 0) != ncBad) {
@@ -973,7 +1062,6 @@ NcBool NcVar::sync(void)
 	the_name = 0;
 	return FALSE;
     }
-    cur_rec = 0;
     init_cur(); 
     return TRUE;
 }
@@ -990,7 +1078,6 @@ NcVar::NcVar(NcFile* nc, int id)
     } else {
 	the_name = 0;
     }
-    cur_rec = 0;
     init_cur();
 }
 
@@ -1021,8 +1108,9 @@ NcToken NcVar::attname( int attnum ) const // caller must delete[]
 void NcVar::init_cur( void )
 {
     the_cur = new long[MAX_NC_DIMS]; // *** don't know num_dims() yet?
-    for(int i = 0; i < MAX_NC_DIMS; i++)
-      the_cur[i] = 0;
+    cur_rec = new long[MAX_NC_DIMS]; // *** don't know num_dims() yet?
+    for(int i = 0; i < MAX_NC_DIMS; i++) { 
+	the_cur[i] = 0; cur_rec[i] = 0; }
 }
 
 NcAtt::NcAtt(NcFile* nc, const NcVar* var, NcToken name)
