@@ -2,7 +2,7 @@
  *	Copyright 1996, University Corporation for Atmospheric Research
  *	See netcdf/COPYRIGHT file for copying and redistribution conditions.
  */
-/* $Id: ffio.c,v 1.49 1998/10/02 17:13:06 davis Exp $ */
+/* $Id: ffio.c,v 1.55 2006/01/03 04:54:00 russ Exp $ */
 /* addition by O. Heudecker, AWI-Bremerhaven, 12.3.1998 */
 /* added correction by John Sheldon and Hans Vahlenkamp 15.4.1998*/
 
@@ -85,6 +85,37 @@ fgrow(const int fd, const off_t len)
 	return ENOERR;
 }
 
+
+/*
+ * Sortof like ftruncate, except won't make the file shorter.  Differs
+ * from fgrow by only writing one byte at designated seek position, if
+ * needed.
+ */
+static int
+fgrow2(const int fd, const off_t len)
+{
+	struct ffc_stat_s sb;
+	struct ffsw sw;
+	if (fffcntl(fd, FC_STAT, &sb, &sw) < 0)
+		return errno;
+	if (len <= sb.st_size)
+		return ENOERR;
+	{
+	    const char dumb = 0;
+	    /* we don't use ftruncate() due to problem with FAT32 file systems */
+	    /* cache current position */
+	    const off_t pos = lseek(fd, 0, SEEK_CUR);
+	    if(pos < 0)
+		return errno;
+	    if (lseek(fd, len-1, SEEK_SET) < 0)
+		return errno;
+	    if(write(fd, &dumb, sizeof(dumb)) < 0)
+		return errno;
+	    if (lseek(fd, pos, SEEK_SET) < 0)
+		return errno;
+	}
+	return ENOERR;
+}
 /* End OS */
 /* Begin ffio */
 
@@ -213,7 +244,6 @@ ncio_ffio_get(ncio *const nciop,
 
 	assert(extent != 0);
 	assert(extent < X_INT_MAX); /* sanity check */
-	assert(offset < X_INT_MAX); /* sanity check */
 
 	assert(ffp->bf_cnt == 0);
 
@@ -504,7 +534,7 @@ ncio_ffio_assign(const char *filename) {
 	if (istat == 0) {	/* user has already specified an assign */
 		return buffer;
 	} else if (istat > 0 || istat < -1) {	/* error occured */
-		errno = NC_EINVAL;		/* as good as any */
+		errno = EINVAL;
 		return (const char *) NULL;
 	} /* istat = -1 -> no assign for file */
 	envstr = getenv("NETCDF_FFIOSPEC");
@@ -531,7 +561,7 @@ ncio_ffio_assign(const char *filename) {
 	if (istat == 0) {	/* success */
 		return buffer;
 	} else {		/* error */
-		errno = NC_EINVAL;
+		errno = EINVAL;
 		return (const char *) NULL;
 	}
 }
@@ -722,9 +752,67 @@ unwind_new:
 }
 
 
+/* 
+ * Get file size in bytes.  
+ * Is use of ffseek() really necessary, or could we use standard fstat() call
+ * and get st_size member?
+ */
+int
+ncio_filesize(ncio *nciop, off_t *filesizep)
+{
+    off_t filesize, current, reset;
+
+    if(nciop == NULL)
+	return EINVAL;
+
+    current = ffseek(nciop->fd, 0, SEEK_CUR);  /* save current */
+    *filesizep = ffseek(nciop->fd, 0, SEEK_END); /* get size */
+    reset = ffseek(nciop->fd, current, SEEK_SET); /* reset */ 
+
+    if(reset != current)
+	return EINVAL;
+    return ENOERR;
+}
+
+
+/*
+ * Sync any changes to disk, then extend file so its size is length.
+ * This is only intended to be called before close, if the file is
+ * open for writing and the actual size does not match the calculated
+ * size, perhaps as the result of having been previously written in
+ * NOFILL mode.
+ */
+int
+ncio_pad_length(ncio *nciop, off_t length)
+{
+	int status = ENOERR;
+
+	if(nciop == NULL)
+		return EINVAL;
+
+	if(!fIsSet(nciop->ioflags, NC_WRITE))
+	        return EPERM; /* attempt to write readonly file */
+
+	status = nciop->sync(nciop);
+	if(status != ENOERR)
+	        return status;
+
+	status = fgrow2(nciop->fd, length);
+	if(status != ENOERR)
+	        return errno;
+	return ENOERR;
+}
+
+
 int 
 ncio_close(ncio *nciop, int doUnlink)
 {
+	/*
+         * TODO: I believe this function is lacking the de-assignment of the
+         * Fortran LUN assigned by ASNFILE in ncio_ffio_assign(...) -- SRE
+         * 2002-07-10.
+	 */
+
 	int status = ENOERR;
 
 	if(nciop == NULL)
