@@ -2,7 +2,7 @@
  *	Copyright 1996, University Corporation for Atmospheric Research
  *	See netcdf/COPYRIGHT file for copying and redistribution conditions.
  */
-/* "$Id: ncx.h,v 1.53 2000/04/05 19:36:56 steve Exp $" */
+/* "$Id: ncx.h,v 1.58 2005/02/07 03:56:02 ed Exp $" */
 
 #ifndef _NCX_H_
 #define _NCX_H_
@@ -22,7 +22,7 @@
  * 
  */
 
-#include "ncconfig.h" /* output of 'configure' */
+#include <config.h> /* output of 'configure' */
 #include "rnd.h"
 #include <stddef.h> /* size_t */
 #include <errno.h>
@@ -34,6 +34,13 @@
 #define CRAYFLOAT 1 /* CRAY Floating point */
 #endif
 
+
+#if defined(DLL_NETCDF) /* define when library is a DLL */
+#include <io.h>
+#define lseek _lseeki64
+#define off_t __int64
+#endif	/* defined(DLL_NETCDF) */
+
 /*
  * The integer return code for the conversion routines
  * is 0 (ENOERR) when no error occured, or NC_ERANGE as appropriate
@@ -63,10 +70,13 @@
 #define X_SIZEOF_DOUBLE		8
 
 /*
- * For now, netcdf is limited to 32 bit offsets and sizes,
+ * For now, netcdf is limited to 32 bit sizes,
+ * If compiled with support for "large files", then
+ * netcdf will use a 64 bit off_t and it can then write a file
+ * using 64 bit offsets. 
  *  see also X_SIZE_MAX, X_OFF_MAX below
  */
-#define X_SIZEOF_OFF_T		X_SIZEOF_INT
+#define X_SIZEOF_OFF_T		(sizeof(off_t))
 #define X_SIZEOF_SIZE_T		X_SIZEOF_INT
 
 /*
@@ -84,12 +94,7 @@
 #define X_INT_MIN	(-2147483647-1)
 #define X_INT_MAX	2147483647
 #define X_UINT_MAX	4294967295U
-#if 0
-#define X_LONG_MIN	(-2147483647-1)
-#define X_LONG_MAX	2147483647
-#define X_ULONG_MAX	4294967295U
-#endif
-#define X_FLOAT_MAX	3.40282347e+38f
+#define X_FLOAT_MAX	3.402823466e+38f
 #define X_FLOAT_MIN	(-X_FLOAT_MAX)
 #define X_FLT_MAX	X_FLOAT_MAX	/* alias compatible with limits.h */
 #if CRAYFLOAT
@@ -102,7 +107,7 @@
 #define X_DOUBLE_MIN	(-X_DOUBLE_MAX)
 #define X_DBL_MAX	X_DOUBLE_MAX	/* alias compatible with limits.h */
 
-#define X_SIZE_MAX	X_INT_MAX	/* N.B., just uses the signed range */
+#define X_SIZE_MAX	X_UINT_MAX
 #define X_OFF_MAX	X_INT_MAX
 
 
@@ -361,14 +366,14 @@ extern int
 ncx_get_size_t(const void **xpp, size_t *ulp);
 /* ncx_get_int_off_t */
 extern int
-ncx_get_off_t(const void **xpp, off_t *lp);
+ncx_get_off_t(const void **xpp, off_t *lp, size_t sizeof_off_t);
 
 /* ncx_put_int_size_t */
 extern int
 ncx_put_size_t(void **xpp, const size_t *ulp);
 /* ncx_put_int_off_t */
 extern int
-ncx_put_off_t(void **xpp, const off_t *lp);
+ncx_put_off_t(void **xpp, const off_t *lp, size_t sizeof_off_t);
 
 
 /*
