@@ -1,7 +1,7 @@
 /*********************************************************************
  *   Copyright 1993, UCAR/Unidata
  *   See netcdf/COPYRIGHT file for copying and redistribution conditions.
- *   $Header: /upc/share/CVS/netcdf-3/ncdump/vardata.c,v 1.7 1998/03/03 00:43:17 davis Exp $
+ *   $Header: /upc/share/CVS/netcdf-3/ncdump/vardata.c,v 1.12 2005/07/22 23:01:39 russ Exp $
  *********************************************************************/
 
 #include <stdio.h>
@@ -20,37 +20,37 @@
 static float float_epsilon(void);
 static double double_epsilon(void);
 static void init_epsilons(void);
-static void printbval(char* sout, const char* fmt, const struct ncvar* varp,
+static void printbval(char* sout, const char* fmt, const ncvar_t* varp,
 		      signed char val);
-static void printsval(char* sout, const char* fmt, const struct ncvar* varp,
+static void printsval(char* sout, const char* fmt, const ncvar_t* varp,
 		      short val);
-static void printival(char* sout, const char* fmt, const struct ncvar* varp,
+static void printival(char* sout, const char* fmt, const ncvar_t* varp,
 		      int val);
-static void printfval(char* sout, const char* fmt, const struct ncvar* varp,
+static void printfval(char* sout, const char* fmt, const ncvar_t* varp,
 		      float val);
-static void printdval(char* sout, const char* fmt, const struct ncvar* varp,
+static void printdval(char* sout, const char* fmt, const ncvar_t* varp,
 		      double val);
 static void lastdelim(boolean  more, boolean lastrow);
-static void annotate(const struct ncvar* vp, const struct fspec* fsp,
+static void annotate(const ncvar_t* vp, const fspec_t* fsp,
 		     const size_t* cor, long iel);
-static void pr_tvals(const struct ncvar *vp, size_t len, const char *fmt,
+static void pr_tvals(const ncvar_t *vp, size_t len, const char *fmt,
 		     boolean more, boolean lastrow, const char *vals,
-		     const struct fspec* fsp, const size_t *cor);
-static void pr_bvals(const struct ncvar *vp, size_t len, const char *fmt,
+		     const fspec_t* fsp, const size_t *cor);
+static void pr_bvals(const ncvar_t *vp, size_t len, const char *fmt,
 		     boolean more, boolean lastrow, const signed char *vals,
-		     const struct fspec* fsp, const size_t *cor);
-static void pr_svals(const struct ncvar *vp, size_t len, const char *fmt,
+		     const fspec_t* fsp, const size_t *cor);
+static void pr_svals(const ncvar_t *vp, size_t len, const char *fmt,
 		     boolean more, boolean lastrow, const short *vals,
-		     const struct fspec* fsp, const size_t *cor);
-static void pr_ivals(const struct ncvar *vp, size_t len, const char *fmt,
+		     const fspec_t* fsp, const size_t *cor);
+static void pr_ivals(const ncvar_t *vp, size_t len, const char *fmt,
 		     boolean more, boolean lastrow, const int *vals,
-		     const struct fspec* fsp, const size_t *cor);
-static void pr_fvals(const struct ncvar *vp, size_t len, const char *fmt,
+		     const fspec_t* fsp, const size_t *cor);
+static void pr_fvals(const ncvar_t *vp, size_t len, const char *fmt,
 		     boolean more, boolean lastrow, const float *vals,
-		     const struct fspec* fsp, const size_t *cor);
-static void pr_dvals(const struct ncvar *vp, size_t len, const char *fmt,
+		     const fspec_t* fsp, const size_t *cor);
+static void pr_dvals(const ncvar_t *vp, size_t len, const char *fmt,
 		     boolean more, boolean lastrow, const double *vals,
-		     const struct fspec* fsp, const size_t *cor);
+		     const fspec_t* fsp, const size_t *cor);
 static int  upcorner(const size_t* dims, int ndims, size_t* odom,
 		     const size_t* add);
 static void lastdelim2 (boolean more, boolean lastrow);
@@ -138,7 +138,7 @@ static void
 printbval(
     char *sout,			/* string where output goes */
     const char *fmt,		/* printf format used for value */
-    const struct ncvar *varp,	/* variable */
+    const ncvar_t *varp,	/* variable */
     signed char val		/* value */
     )
 {
@@ -161,7 +161,7 @@ static void
 printsval(
     char *sout,			/* string where output goes */
     const char *fmt,		/* printf format used for value */
-    const struct ncvar *varp,		/* variable */
+    const ncvar_t *varp,		/* variable */
     short val			/* value */
     )
 {
@@ -185,7 +185,7 @@ static void
 printival(
     char *sout,			/* string where output goes */
     const char *fmt,		/* printf format used for value */
-    const struct ncvar *varp,		/* variable */
+    const ncvar_t *varp,		/* variable */
     int val			/* value */
     )
 {
@@ -212,7 +212,7 @@ static void
 printfval(
     char *sout,			/* string where output goes */
     const char *fmt,		/* printf format used for value */
-    const struct ncvar *varp,		/* variable */
+    const ncvar_t *varp,		/* variable */
     float val			/* value */
     )
 {
@@ -238,7 +238,7 @@ static void
 printdval(
     char *sout,			/* string where output goes */
     const char *fmt,		/* printf format used for value */
-    const struct ncvar *varp,		/* variable */
+    const ncvar_t *varp,		/* variable */
     double val			/* value */
     )
 {
@@ -296,8 +296,8 @@ lastdelim2 (boolean more, boolean lastrow)
  */
 static void
 annotate(
-     const struct ncvar *vp,	/* variable */
-     const struct fspec* fsp,	/* formatting specs */
+     const ncvar_t *vp,	/* variable */
+     const fspec_t* fsp,	/* formatting specs */
      const size_t *cor,		/* corner coordinates */
      long iel			/* which element in current row */
      )
@@ -332,7 +332,7 @@ annotate(
  */
 static void
 pr_tvals(
-     const struct ncvar *vp,		/* variable */
+     const ncvar_t *vp,		/* variable */
      size_t len,		/* number of values to print */
      const char *fmt,		/* printf format used for each value.  If
 				 * nc_type is NC_CHAR and this is NULL,
@@ -344,7 +344,7 @@ pr_tvals(
 				 * variable, so terminate with ";" instead
 				 * of "," */
      const char *vals,		/* pointer to block of values */
-     const struct fspec* fsp,	/* formatting specs */
+     const fspec_t* fsp,          /* formatting specs */
      const size_t *cor		/* corner coordinates */
      )
 {
@@ -397,7 +397,6 @@ pr_tvals(
 	    }
 	Printf("\"");
 	if (fsp->full_data_cmnts) {
-	    Printf("\"");
 	    lastdelim (more, lastrow);
 	    annotate (vp, fsp,  (size_t *)cor, 0L);
 	}
@@ -434,7 +433,7 @@ pr_tvals(
  */
 static void
 pr_bvals(
-     const struct ncvar *vp,	/* variable */
+     const ncvar_t *vp,	/* variable */
      size_t len,		/* number of values to print */
      const char *fmt,		/* printf format used for each value.  If
 				 * nc_type is NC_CHAR and this is NULL,
@@ -446,7 +445,7 @@ pr_bvals(
 				 * variable, so terminate with ";" instead
 				 * of "," */
      const signed char *vals,	/* pointer to block of values */
-     const struct fspec* fsp,	/* formatting specs */
+     const fspec_t* fsp,	        /* formatting specs */
      const size_t *cor		/* corner coordinates */
      )
 {
@@ -482,7 +481,7 @@ pr_bvals(
  */
 static void
 pr_svals(
-     const struct ncvar *vp,		/* variable */
+     const ncvar_t *vp,		/* variable */
      size_t len,		/* number of values to print */
      const char *fmt,		/* printf format used for each value.  If
 				 * nc_type is NC_CHAR and this is NULL,
@@ -494,7 +493,7 @@ pr_svals(
 				 * variable, so terminate with ";" instead
 				 * of "," */
      const short *vals,		/* pointer to block of values */
-     const struct fspec* fsp,	/* formatting specs */
+     const fspec_t* fsp,	        /* formatting specs */
      const size_t *cor		/* corner coordinates */
      )
 {
@@ -532,7 +531,7 @@ pr_svals(
  */
 static void
 pr_ivals(
-     const struct ncvar *vp,		/* variable */
+     const ncvar_t *vp,		/* variable */
      size_t len,		/* number of values to print */
      const char *fmt,		/* printf format used for each value.  If
 				 * nc_type is NC_CHAR and this is NULL,
@@ -544,7 +543,7 @@ pr_ivals(
 				 * variable, so terminate with ";" instead
 				 * of "," */
      const int *vals,		/* pointer to block of values */
-     const struct fspec* fsp,	/* formatting specs */
+     const fspec_t* fsp,	        /* formatting specs */
      const size_t *cor		/* corner coordinates */
      )
 {
@@ -580,7 +579,7 @@ pr_ivals(
  */
 static void
 pr_fvals(
-     const struct ncvar *vp,		/* variable */
+     const ncvar_t *vp,		/* variable */
      size_t len,			/* number of values to print */
      const char *fmt,		/* printf format used for each value.  If
 				 * nc_type is NC_CHAR and this is NULL,
@@ -592,7 +591,7 @@ pr_fvals(
 				 * variable, so terminate with ";" instead
 				 * of "," */
      const float *vals,		/* pointer to block of values */
-     const struct fspec* fsp,	/* formatting specs */
+     const fspec_t* fsp,	        /* formatting specs */
      const size_t *cor		/* corner coordinates */
      )
 {
@@ -628,7 +627,7 @@ pr_fvals(
  */
 static void
 pr_dvals(
-     const struct ncvar *vp,		/* variable */
+     const ncvar_t *vp,		/* variable */
      size_t len,			/* number of values to print */
      const char *fmt,		/* printf format used for each value.  If
 				 * nc_type is NC_CHAR and this is NULL,
@@ -640,7 +639,7 @@ pr_dvals(
 				 * variable, so terminate with ";" instead
 				 * of "," */
      const double *vals,	/* pointer to block of values */
-     const struct fspec* fsp,	/* formatting specs */
+     const fspec_t* fsp,	        /* formatting specs */
      const size_t *cor		/* corner coordinates */
      )
 {
@@ -702,11 +701,11 @@ upcorner(
 /* Output the data for a single variable, in CDL syntax. */
 int
 vardata(
-     const struct ncvar *vp,	/* variable */
+     const ncvar_t *vp,	/* variable */
      size_t vdims[],		/* variable dimension sizes */
      int ncid,			/* netcdf id */
      int varid,			/* variable id */
-     const struct fspec* fsp	/* formatting specs */
+     const fspec_t* fsp	        /* formatting specs */
      )
 {
     size_t cor[NC_MAX_DIMS];	/* corner coordinates */
@@ -726,7 +725,7 @@ vardata(
     static int initeps = 0;
 
     /* printf format used to print each value */
-    char *fmt = get_fmt(ncid, varid, vp->type);
+    const char *fmt = get_fmt(ncid, varid, vp->type);
 
     if (!initeps) {		/* make sure epsilons get initialized */
 	init_epsilons();
@@ -860,3 +859,429 @@ vardata(
 
     return 0;
 }
+
+
+/*
+ * print last delimiter in each line before annotation (, or ;)
+ */
+static void
+lastdelim2x (boolean more, boolean lastrow)
+{
+    if (more) {
+	lput(" ");
+    } else {
+	if(lastrow) {
+	    lput("\n   ");
+	} else {
+	    lput("\n     ");
+	}
+    }
+}
+
+
+/*
+ * Print a number of char variable values.
+ */
+static void
+pr_tvalsx(
+     size_t len,		/* number of values to print */
+     const char *fmt,		/* printf format used for each value.  If
+				 * nc_type is NC_CHAR and this is NULL,
+				 * character arrays will be printed as
+				 * strings enclosed in quotes.  */
+     boolean more,		/* true if more data for this row will
+				 * follow, so add trailing comma */
+     boolean lastrow,		/* true if this is the last row for this
+				 * variable, so terminate with ";" instead
+				 * of "," */
+     const char *vals,		/* pointer to block of values */
+     const fspec_t *fsp	        /* formatting specs */
+     )
+{
+    long iel;
+    const char *sp;
+    unsigned char uc;
+    char sout[100];		/* temporary string for each encoded output */
+
+    if (fmt == 0 || STREQ(fmt,"%s") || STREQ(fmt,"")) { /* as string */
+	Printf("\"");
+	/* adjust len so trailing nulls don't get printed */
+	sp = vals + len;
+	while (len != 0 && *--sp == '\0')
+	    len--;
+	for (iel = 0; iel < len; iel++)
+	    switch (uc = *vals++ & 0377) {
+	    case '\b':
+		Printf("\\b");
+		break;
+	    case '\f':
+		Printf("\\f");
+		break;
+	    case '\n':	/* generate linebreaks after new-lines */
+		Printf("\\n\",\n    \"");
+		break;
+	    case '\r':
+		Printf("\\r");
+		break;
+	    case '\t':
+		Printf("\\t");
+		break;
+	    case '\v':
+		Printf("\\v");
+		break;
+	    case '\\':
+		Printf("\\\\");
+		break;
+	    case '\'':
+		Printf("\\\'");
+		break;
+	    case '\"':
+		Printf("\\\"");
+		break;
+	    default:
+		if (isprint(uc))
+		    Printf("%c",uc);
+		else
+		    Printf("\\%.3o",uc);
+		break;
+	    }
+	Printf("\"");
+    } else {		/* use format from C_format attribute */
+	for (iel = 0; iel < len-1; iel++) {
+	    (void) sprintf(sout, fmt, *vals++);
+	    (void) strcat(sout, " ");
+	    lput(sout);
+	}
+	(void) sprintf(sout, fmt, *vals++);
+	lput(sout);
+    }
+    if (!fsp->full_data_cmnts) {
+	lastdelim2x (more, lastrow);
+    }
+}
+
+
+/*
+ * Print a number of byte variable values.
+ */
+static void
+pr_bvalsx(
+     const ncvar_t *vp,	/* variable */
+     size_t len,		/* number of values to print */
+     const char *fmt,		/* printf format used for each value.  If
+				 * nc_type is NC_CHAR and this is NULL,
+				 * character arrays will be printed as
+				 * strings enclosed in quotes.  */
+     boolean more,		/* true if more data for this row will
+				 * follow, so add trailing comma */
+     boolean lastrow,		/* true if this is the last row for this
+				 * variable, so terminate with ";" instead
+				 * of "," */
+     const signed char *vals	/* pointer to block of values */
+     )
+{
+    long iel;
+    char sout[100];		/* temporary string for each encoded output */
+
+    for (iel = 0; iel < len-1; iel++) {
+	printbval(sout, fmt, vp, *vals++);
+	(void) strcat(sout, " ");
+	lput(sout);
+    }
+    printbval(sout, fmt, vp, *vals++);
+    lput(sout);
+    lastdelim2x (more, lastrow);
+}
+
+
+/*
+ * Print a number of short variable values.
+ */
+static void
+pr_svalsx(
+     const ncvar_t *vp,		/* variable */
+     size_t len,		/* number of values to print */
+     const char *fmt,		/* printf format used for each value.  If
+				 * nc_type is NC_CHAR and this is NULL,
+				 * character arrays will be printed as
+				 * strings enclosed in quotes.  */
+     boolean more,		/* true if more data for this row will
+				 * follow, so add trailing comma */
+     boolean lastrow,		/* true if this is the last row for this
+				 * variable, so terminate with ";" instead
+				 * of "," */
+     const short *vals		/* pointer to block of values */
+     )
+{
+    long iel;
+    char sout[100];		/* temporary string for each encoded output */
+
+    for (iel = 0; iel < len-1; iel++) {
+	printsval(sout, fmt, vp, *vals++);
+	(void) strcat(sout, " ");
+	lput(sout);
+    }
+    printsval(sout, fmt, vp, *vals++);
+    lput(sout);
+    lastdelim2x (more, lastrow);
+}
+
+
+
+
+/*
+ * Print a number of int variable values.
+ */
+static void
+pr_ivalsx(
+     const ncvar_t *vp,		/* variable */
+     size_t len,		/* number of values to print */
+     const char *fmt,		/* printf format used for each value.  If
+				 * nc_type is NC_CHAR and this is NULL,
+				 * character arrays will be printed as
+				 * strings enclosed in quotes.  */
+     boolean more,		/* true if more data for this row will
+				 * follow, so add trailing comma */
+     boolean lastrow,		/* true if this is the last row for this
+				 * variable, so terminate with ";" instead
+				 * of "," */
+     const int *vals		/* pointer to block of values */
+     )
+{
+    long iel;
+    char sout[100];		/* temporary string for each encoded output */
+
+    for (iel = 0; iel < len-1; iel++) {
+	printival(sout, fmt, vp, *vals++);
+	(void) strcat(sout, " ");
+	lput(sout);
+    }
+    printival(sout, fmt, vp, *vals++);
+    lput(sout);
+    lastdelim2x (more, lastrow);
+}
+
+
+/*
+ * Print a number of float variable values.
+ */
+static void
+pr_fvalsx(
+     const ncvar_t *vp,		/* variable */
+     size_t len,			/* number of values to print */
+     const char *fmt,		/* printf format used for each value.  If
+				 * nc_type is NC_CHAR and this is NULL,
+				 * character arrays will be printed as
+				 * strings enclosed in quotes.  */
+     boolean more,		/* true if more data for this row will
+				 * follow, so add trailing comma */
+     boolean lastrow,		/* true if this is the last row for this
+				 * variable, so terminate with ";" instead
+				 * of "," */
+     const float *vals		/* pointer to block of values */
+     )
+{
+    long iel;
+    char sout[100];		/* temporary string for each encoded output */
+
+    for (iel = 0; iel < len-1; iel++) {
+	printfval(sout, fmt, vp, *vals++);
+	(void) strcat(sout, " ");
+	lput(sout);
+    }
+    printfval(sout, fmt, vp, *vals++);
+    lput(sout);
+    lastdelim2x (more, lastrow);
+}
+
+
+/*
+ * Print a number of double variable values.
+ */
+static void
+pr_dvalsx(
+     const ncvar_t *vp,		/* variable */
+     size_t len,			/* number of values to print */
+     const char *fmt,		/* printf format used for each value.  If
+				 * nc_type is NC_CHAR and this is NULL,
+				 * character arrays will be printed as
+				 * strings enclosed in quotes.  */
+     boolean more,		/* true if more data for this row will
+				 * follow, so add trailing comma */
+     boolean lastrow,		/* true if this is the last row for this
+				 * variable, so terminate with ";" instead
+				 * of "," */
+     const double *vals		/* pointer to block of values */
+     )
+{
+    long iel;
+    char sout[100];		/* temporary string for each encoded output */
+
+    for (iel = 0; iel < len-1; iel++) {
+	printdval(sout, fmt, vp, *vals++);
+	(void) strcat(sout, " ");
+	lput(sout);
+    }
+    printdval(sout, fmt, vp, *vals++);
+    lput(sout);
+    lastdelim2x (more, lastrow);
+}
+
+
+/* Output the data for a single variable, in NcML syntax. */
+int
+vardatax(
+     const ncvar_t *vp,	/* variable */
+     size_t vdims[],		/* variable dimension sizes */
+     int ncid,			/* netcdf id */
+     int varid,			/* variable id */
+     const fspec_t *fsp	        /* formatting specs */
+     )
+{
+    size_t cor[NC_MAX_DIMS];	/* corner coordinates */
+    size_t edg[NC_MAX_DIMS];	/* edges of hypercube */
+    size_t add[NC_MAX_DIMS];	/* "odometer" increment to next "row"  */
+#define VALBUFSIZ 1000
+    double vals[VALBUFSIZ] ; /* aligned buffer */
+
+    int gulp = VALBUFSIZ;
+
+    int id;
+    int ir;
+    size_t nels;
+    size_t ncols;
+    size_t nrows;
+    int vrank = vp->ndims;
+    static int initeps = 0;
+
+    /* printf format used to print each value */
+    const char *fmt = get_fmt(ncid, varid, vp->type);
+
+    if (!initeps) {		/* make sure epsilons get initialized */
+	init_epsilons();
+	initeps = 1;
+    }
+
+    nels = 1;
+    for (id = 0; id < vrank; id++) {
+	cor[id] = 0;
+	edg[id] = 1;
+	nels *= vdims[id];	/* total number of values for variable */
+    }
+
+    Printf("    <values>\n     ");
+    set_indent (7);
+
+    if (vrank < 1) {
+	ncols = 1;
+    } else {
+	ncols = vdims[vrank-1];	/* size of "row" along last dimension */
+	edg[vrank-1] = vdims[vrank-1];
+	for (id = 0; id < vrank; id++)
+	  add[id] = 0;
+	if (vrank > 1)
+	  add[vrank-2] = 1;
+    }
+    nrows = nels/ncols;		/* number of "rows" */
+    
+    for (ir = 0; ir < nrows; ir++) {
+	/*
+	 * rather than just printing a whole row at once (which might exceed
+	 * the capacity of MSDOS platforms, for example), we break each row
+	 * into smaller chunks, if necessary.
+	 */
+	size_t corsav;
+	int left = (int)ncols;
+	boolean lastrow;
+
+	if (vrank > 0) {
+	    corsav = cor[vrank-1];
+	}
+	lastrow = (boolean)(ir == nrows-1);
+	while (left > 0) {
+	    size_t toget = left < gulp ? left : gulp;
+	    if (vrank > 0)
+	      edg[vrank-1] = toget;
+	    switch(vp->type) {
+	    case NC_CHAR:
+		NC_CHECK(
+		    nc_get_vara_text(ncid, varid, cor, edg, (char *)vals) );
+	        pr_tvalsx(toget, fmt, left > toget, lastrow,
+			 (char *) vals, fsp);
+		break;
+	    case NC_BYTE:
+		NC_CHECK(
+		    nc_get_vara_schar(ncid, varid, cor, edg, (signed char *)vals) );
+	        pr_bvalsx(vp, toget, fmt, left > toget, lastrow,
+			 (signed char *) vals);
+		break;
+	    case NC_SHORT:
+		NC_CHECK(
+		    nc_get_vara_short(ncid, varid, cor, edg, (short *)vals) );
+	        pr_svalsx(vp, toget, fmt, left > toget, lastrow,
+			 (short *) vals);
+		break;
+	    case NC_INT:
+		NC_CHECK(
+		    nc_get_vara_int(ncid, varid, cor, edg, (int *)vals) );
+	        pr_ivalsx(vp, toget, fmt, left > toget, lastrow,
+			 (int *) vals);
+		break;
+	    case NC_FLOAT:
+		NC_CHECK(
+		    nc_get_vara_float(ncid, varid, cor, edg, (float *)vals) );
+	        pr_fvalsx(vp, toget, fmt, left > toget, lastrow,
+			 (float *) vals);
+		break;
+	    case NC_DOUBLE:
+		NC_CHECK(
+		    nc_get_vara_double(ncid, varid, cor, edg, (double *)vals) );
+	        pr_dvalsx(vp, toget, fmt, left > toget, lastrow,
+			 (double *) vals);
+		break;
+#ifdef USE_NETCDF4
+	    case NC_UBYTE:
+		/* TODO */
+		break;
+	    case NC_USHORT:
+		/* TODO */
+		break;
+	    case NC_UINT:
+		/* TODO */
+		break;
+	    case NC_INT64:
+		/* TODO */
+		break;
+	    case NC_UINT64:
+		/* TODO */
+		break;
+	    case NC_STRING:
+		/* TODO */
+		break;
+	    case NC_VLEN:
+		/* TODO */
+		break;
+	    case NC_OPAQUE:
+		/* TODO */
+		break;
+	    case NC_COMPOUND:
+		/* TODO */
+		break;
+#endif /* USE_NETCDF4 */
+	    default:
+		error("vardata: bad type");
+	    }
+	    left -= toget;
+	    if (vrank > 0)
+	      cor[vrank-1] += toget;
+	}
+	if (vrank > 0)
+	  cor[vrank-1] = corsav;
+	if (ir < nrows-1)
+	  if (!upcorner(vdims,vp->ndims,cor,add))
+	    error("vardata: odometer overflowed!");
+	set_indent(2);
+    }
+    Printf(" </values>\n");
+    return 0;
+}
