@@ -1,7 +1,7 @@
 /*********************************************************************
  *   Copyright 1993, UCAR/Unidata
  *   See netcdf/COPYRIGHT file for copying and redistribution conditions.
- *   $Header: /upc/share/CVS/netcdf-3/nctest/dimtests.c,v 1.11 1996/04/30 17:56:56 davis Exp $
+ *   $Header: /upc/share/CVS/netcdf-3/nctest/dimtests.c,v 1.13 2005/03/17 14:59:18 ed Exp $
  *********************************************************************/
 
 #include <stdio.h>
@@ -23,9 +23,9 @@
  *    make sure unlimited size works, shows up in ncinquire(...,*xtendim)
  *    try to define a second unlimited dimension, check error
  */
-void
+int
 test_ncdimdef(path)
-     char *path;		/* name of writable netcdf to open */
+     const char *path;		/* name of writable netcdf to open */
 {
     int nerrs = 0;
     static char pname[] = "test_ncdimdef";
@@ -46,50 +46,50 @@ test_ncdimdef(path)
 
     if ((cdfid = ncopen(path, NC_WRITE)) == -1) {
 	error("%s: ncopen failed", pname);
-	return;
+	return ++nerrs;
     }
     /* opened, defining a dimension should fail in data mode */
     if (ncdimdef(cdfid, mm.name, mm.size) != -1) {
 	error("%s: ncdimdef should have failed in data mode", pname);
-	ncclose(cdfid); return;
+	ncclose(cdfid); return ++nerrs;
     }
     /* enter define mode */
     if (ncredef(cdfid) == -1) {
 	error("%s: cdredef failed", pname);
-	ncclose(cdfid); return;
+	ncclose(cdfid); return ++nerrs;
     }
     /* in define mode OK, add a dimension */
     if ((dimid = ncdimdef(cdfid, mm.name, mm.size)) == -1) {
 	error("%s: ncdimdef failed", pname);
-	ncclose(cdfid); return;
+	ncclose(cdfid); return ++nerrs;
     }
     add_dim(&test, &mm);	/* keep in-memory netcdf in sync */
     /* check that dim id returned is one more than previous dim id */
     if (dimid != test.ndims - 1) {
 	error("%s: ncdimdef returned %d for dim id, expected %d",
 	      pname, dimid, test.ndims-1);
-	ncclose(cdfid); return;
+	ncclose(cdfid); return ++nerrs;
     }
 
     /* try adding same dimension again, this should fail */
     if (ncdimdef(cdfid, mm.name, mm.size) != -1) {
 	error("%s: ncdimdef should not have allowed redefinition", pname);
-	ncclose(cdfid); return;
+	ncclose(cdfid); return ++nerrs;
     }
     /* try adding dimension with negative size, this should fail */
     if (ncdimdef(cdfid, nn.name, (long) -10) != -1) {
 	error("%s: ncdimdef should not allow negative size dimension", pname);
-	ncclose(cdfid); return;
+	ncclose(cdfid); return ++nerrs;
     }
     /* if there is not already an unlimited size dimension, try adding one */
     if (ncinquire(cdfid, &ndims, &nvars, &natts, &xdimid) == -1) {
 	error("%s: ncinquire failed", pname);
-	ncclose(cdfid); return;
+	ncclose(cdfid); return ++nerrs;
     }
     if (xdimid == -1) {
 	if (ncdimdef(cdfid, rec.name, rec.size) == -1) {
 	    error("%s: ncdimdef failed on NC_UNLIMITED dimension", pname);
-	    ncclose(cdfid); return;
+	    ncclose(cdfid); return ++nerrs;
 	}
 	add_dim(&test, &rec);
     }
@@ -97,15 +97,15 @@ test_ncdimdef(path)
     if (ncdimdef(cdfid, "rec2", rec.size) != -1) {
 	error("%s: ncdimdef should not allow second NC_UNLIMITED dimension",
 	      pname);
-	ncclose(cdfid); return;
+	ncclose(cdfid); return ++nerrs;
     }
     if (ncendef (cdfid) == -1) {
 	error("%s: ncendef failed", pname);
-	ncclose(cdfid); return;
+	ncclose(cdfid); return ++nerrs;
     }
     if (ncclose (cdfid) == -1) {
 	error("%s: ncclose failed", pname);
-	return;
+	return ++nerrs;
     }
     if (ncdimdef(cdfid, "rec2", rec.size) != -1) {
 	error("%s: ncdimdef should fail on bad netCDF id", pname);
@@ -115,6 +115,8 @@ test_ncdimdef(path)
       (void) fprintf(stderr,"FAILED! ***\n");
     else
       (void) fprintf(stderr,"ok ***\n");
+
+    return nerrs;
 }
 
 
@@ -125,9 +127,9 @@ test_ncdimdef(path)
  *    check return with unlimited size dimension
  *    try with bad handle, check error
  */
-void
+int
 test_ncdimid(path)
-     char *path;		/* name of writable netcdf file to open */
+     const char *path;		/* name of writable netcdf file to open */
 {
     int nerrs = 0;
     static char pname[] = "test_ncdimid";
@@ -140,46 +142,46 @@ test_ncdimid(path)
 
     if ((cdfid = ncopen(path, NC_WRITE)) == -1) {
 	error("%s: ncopen failed", pname);
-	return;
+	return ++nerrs;
     }
     /* opened, enter define mode */
     if (ncredef(cdfid) == -1) {
 	error("%s: cdredef failed", pname);
-	ncclose(cdfid); return;
+	ncclose(cdfid); return ++nerrs;
     }
     /* in define mode OK, add a dimension */
     if ((nn_dim = ncdimdef(cdfid, nn.name, nn.size)) == -1) {
 	error("%s: ncdimdef failed", pname);
-	ncclose(cdfid); return;
+	ncclose(cdfid); return ++nerrs;
     }
     add_dim(&test, &nn);	/* keep in-memory netcdf in sync */
     /* check id returned for name matches id returned from definition */
     if (ncdimid(cdfid, nn.name) != nn_dim) {
 	error("%s: ncdimid returned wrong value in define mode", pname);
-	ncclose(cdfid); return;
+	ncclose(cdfid); return ++nerrs;
     }
     if (ncendef (cdfid) == -1) {
 	error("%s: ncendef failed", pname);
-	ncclose(cdfid); return;
+	ncclose(cdfid); return ++nerrs;
     }
     /* in data mode, check returned id for dimension just added */
     if (ncdimid(cdfid, nn.name) != nn_dim) {
 	error("%s: ncdimid returned wrong value in data mode", pname);
-	ncclose(cdfid); return;
+	ncclose(cdfid); return ++nerrs;
     }
     /* try with undefined dimension, should fail */
     if (ncdimid(cdfid, "easter-bunny") != -1) {
 	error("%s: ncdimid with bogus name should have failed ", pname);
-	ncclose(cdfid); return;
+	ncclose(cdfid); return ++nerrs;
     }
     /* try with unlimited dimension, assumed to be "rec" from earlier calls */
     if (ncdimid(cdfid, "rec") != test.xdimid) {
 	error("%s: ncdimid returned bad value for record dimension", pname);
-	ncclose(cdfid); return;
+	ncclose(cdfid); return ++nerrs;
     }
     if (ncclose (cdfid) == -1) {
 	error("%s: ncclose failed", pname);
-	return;
+	return ++nerrs;
     }
     /* try on bad handle, should fail */
     if (ncdimid(cdfid, nn.name) != -1) {
@@ -190,6 +192,8 @@ test_ncdimid(path)
       (void) fprintf(stderr,"FAILED! ***\n");
     else
       (void) fprintf(stderr,"ok ***\n");
+
+    return nerrs;
 }
 
 
@@ -200,9 +204,9 @@ test_ncdimid(path)
  *    try with bad dimension handle, check error
  *    try with bad netCDF handle, check error
  */
-void
+int
 test_ncdiminq(path)
-     char *path;		/* name of writable netcdf file to open */
+     const char *path;		/* name of writable netcdf file to open */
 {
     int nerrs = 0;
     static char pname[] = "test_ncdiminq";
@@ -214,7 +218,7 @@ test_ncdiminq(path)
 
     if ((cdfid = ncopen(path, NC_WRITE)) == -1) {
 	error("%s: ncopen failed", pname);
-	return;
+	return ++nerrs;
     }
     /* opened, in data mode */
     dim.name = (char *) emalloc(MAX_NC_NAME);
@@ -222,7 +226,7 @@ test_ncdiminq(path)
 	if (ncdiminq(cdfid, dimid, dim.name, &dim.size) == -1) {
 	    error("%s: ncdiminq in data mode failed on dim id %d",
 		  pname, dimid);
-	    ncclose(cdfid); return;
+	    ncclose(cdfid); return ++nerrs;
 	}
 	/* compare returned with expected values */
 	if (strcmp(dim.name, test.dims[dimid].name) != 0) {
@@ -238,14 +242,14 @@ test_ncdiminq(path)
     }
     if (ncredef(cdfid) == -1) {
 	error("%s: ncredef failed", pname);
-	ncclose(cdfid); return;
+	ncclose(cdfid); return ++nerrs;
     }
     /* in define mode, compare returned with expected values again */
     for (dimid = 0 ; dimid < test.ndims; dimid++) { /* loop on all dim ids */
 	if (ncdiminq(cdfid, dimid, dim.name, &dim.size) == -1) {
 	    error("%s: ncdiminq in define mode failed on dim id %d",
 		  pname, dimid);
-	    ncclose(cdfid); return;
+	    ncclose(cdfid); return ++nerrs;
 	}
 	/* compare returned with expected values */
 	if (strcmp(dim.name, test.dims[dimid].name) != 0) {
@@ -264,15 +268,15 @@ test_ncdiminq(path)
 	ncdiminq(cdfid, test.ndims, dim.name, &dim.size) != -1) {
 	error("%s: ncdiminq should have failed on bad dimension ids",
 	      pname, dimid);
-	ncclose(cdfid); return;
+	ncclose(cdfid); return ++nerrs;
     }
     if (ncendef (cdfid) == -1) {
 	error("%s: ncendef failed", pname);
-	ncclose(cdfid); return;
+	ncclose(cdfid); return ++nerrs;
     }
     if (ncclose (cdfid) == -1) {
 	error("%s: ncclose failed", pname);
-	return;
+	return ++nerrs;
     }
     /* should fail, since bad handle */
     if (test.ndims >= 1) {	/* if any dimensions have been defined */
@@ -286,6 +290,8 @@ test_ncdiminq(path)
       (void) fprintf(stderr,"FAILED! ***\n");
     else
       (void) fprintf(stderr,"ok ***\n");
+
+    return nerrs;
 }    
     
 /*
@@ -295,9 +301,9 @@ test_ncdiminq(path)
  *    try with bad dimension handle, check error
  *    try with bad netCDF handle, check error
  */
-void
+int
 test_ncdimrename(path)
-     char *path;		/* name of writable netcdf file to open */
+     const char *path;		/* name of writable netcdf file to open */
 {
     int nerrs = 0;
     static char pname[] = "test_ncdimrename";
@@ -315,43 +321,43 @@ test_ncdimrename(path)
 
     if ((cdfid = ncopen(path, NC_WRITE)) == -1) {
 	error("%s: ncopen failed", pname);
-	return;
+	return ++nerrs;
     }
     /* opened */
     if (ncredef(cdfid) == -1) {
 	error("%s: ncredef failed", pname);
-	ncclose(cdfid); return;
+	ncclose(cdfid); return ++nerrs;
     }
     /* in define mode, add two dimensions */
     if ((pp_dim = ncdimdef(cdfid, pp.name, pp.size)) == -1) {
 	error("%s: ncdimdef failed", pname);
-	ncclose(cdfid); return;
+	ncclose(cdfid); return ++nerrs;
     }
     add_dim(&test, &pp);	/* keep in-memory netcdf in sync */
     if (ncdimdef(cdfid, qq.name, qq.size) == -1) {
 	error("%s: ncdimdef failed", pname);
-	ncclose(cdfid); return;
+	ncclose(cdfid); return ++nerrs;
     }
     add_dim(&test, &qq);	/* keep in-memory netcdf in sync */
     /* rename first dimension */
     if (ncdimrename(cdfid, pp_dim, newname) == -1) {
 	error("%s: ncdimrename failed", pname);
-	ncclose(cdfid); return;
+	ncclose(cdfid); return ++nerrs;
     }
     /* check new name with ncdiminq */
     dim.name = (char *) emalloc(MAX_NC_NAME);
     if (ncdiminq(cdfid, pp_dim, dim.name, &dim.size) == -1) {
 	error("%s: ncdiminq failed", pname);
-	ncclose(cdfid); return;
+	ncclose(cdfid); return ++nerrs;
     }
     if (strcmp(dim.name,pp.name) == 0) {
 	error("%s: ncdimrename failed to change name", pname);
-	ncclose(cdfid); return;
+	ncclose(cdfid); return ++nerrs;
     }
     if (strcmp(dim.name,newname) != 0) {
 	error("%s: ncdimrename changed name to %s instead of %s",
 	      pname, dim.name, newname);
-	ncclose(cdfid); return;
+	ncclose(cdfid); return ++nerrs;
     }
     test.dims[pp_dim].name = (char *) erealloc((void *)test.dims[pp_dim].name,
 					      strlen(newname)+1);
@@ -359,24 +365,24 @@ test_ncdimrename(path)
     /* try to rename second dimension same as first, should fail */
     if (ncdimrename(cdfid, pp_dim, qq.name) != -1) {
 	error("%s: ncdimrename should have failed with used name", pname);
-	ncclose(cdfid); return;
+	ncclose(cdfid); return ++nerrs;
     }
     /* try with bad dimension handles, check for failure */
     if (ncdimrename(cdfid, -1, dim.name) != -1 ||
 	ncdimrename(cdfid, test.ndims, dim.name) != -1) {
 	error("%s: ncdimrename should have failed on bad dimension ids",
 	      pname);
-	ncclose(cdfid); return;
+	ncclose(cdfid); return ++nerrs;
     }
     if (ncendef (cdfid) == -1) {
 	error("%s: ncendef failed", pname);
-	ncclose(cdfid); return;
+	ncclose(cdfid); return ++nerrs;
     }
 
     /* in data mode, rename to shorter name */
     if (ncdimrename(cdfid, pp_dim, "p") == -1) {
 	error("%s: ncdimrename to shorter name failed in data mode", pname);
-	ncclose(cdfid); return;
+	ncclose(cdfid); return ++nerrs;
     }
     test.dims[pp_dim].name = (char *) erealloc((void *)test.dims[pp_dim].name,
 					      strlen("p")+1);
@@ -389,14 +395,14 @@ test_ncdimrename(path)
     /* in data mode, restore old name */
     if (ncdimrename(cdfid, pp_dim, pp.name) == -1) {
 	error("%s: ncdimrename failed in data mode", pname);
-	ncclose(cdfid); return;
+	ncclose(cdfid); return ++nerrs;
     }
     test.dims[pp_dim].name = (char *) erealloc((void *)test.dims[pp_dim].name,
 					      strlen(pp.name)+1);
     (void) strcpy(test.dims[pp_dim].name, pp.name); /* keep test consistent */
     if (ncclose (cdfid) == -1) {
 	error("%s: ncclose failed", pname);
-	return;
+	return ++nerrs;
     }
     /* should fail, since bad handle */
     if (ncdimrename (cdfid, 0, dim.name) != -1) {
@@ -408,4 +414,6 @@ test_ncdimrename(path)
       (void) fprintf(stderr,"FAILED! ***\n");
     else
       (void) fprintf(stderr,"ok ***\n");
+
+    return nerrs;
 }
