@@ -1,7 +1,7 @@
 /*********************************************************************
  *   Copyright 1993, UCAR/Unidata
  *   See netcdf/COPYRIGHT file for copying and redistribution conditions.
- *   $Header: /upc/share/CVS/netcdf-3/nctest/vardef.c,v 1.14 1996/04/30 17:57:02 davis Exp $
+ *   $Header: /upc/share/CVS/netcdf-3/nctest/vardef.c,v 1.16 2005/03/17 14:59:18 ed Exp $
  *********************************************************************/
 
 #include <stdio.h>
@@ -103,9 +103,9 @@ init_epsilons()
  *    try with bad dimension ids, check error
  *    try in data mode, check error
  */
-void
+int
 test_ncvardef(path)
-     char *path;		/* name of writable netcdf file to open */
+     const char *path;		/* name of writable netcdf file to open */
 {
     int nerrs = 0;
     int cdfid;			/* netcdf id */
@@ -135,18 +135,18 @@ test_ncvardef(path)
     init_epsilons();
     if ((cdfid = ncopen(path, NC_WRITE)) == -1) {
 	error("%s: ncopen failed", pname);
-	return;
+	return ++nerrs;
     }
     /* opened, defining a variable should fail in data mode */
     if (ncvardef(cdfid, va[0].name, va[0].type, va[0].ndims, va[0].dims)
 	!= -1) {
 	error("%s: ncvardef should have failed in data mode", pname);
-	ncclose(cdfid); return;
+	ncclose(cdfid); return ++nerrs;
     }
     /* enter define mode */
     if (ncredef(cdfid) == -1) {
 	error("%s: cdredef failed", pname);
-	ncclose(cdfid); return;
+	ncclose(cdfid); return ++nerrs;
     }
 
     /* Add nd more dimensions */
@@ -154,7 +154,7 @@ test_ncvardef(path)
 	if ((di_id[id] = ncdimdef(cdfid, di[id].name, di[id].size)) == -1) {
 	    error("%s: ncdimdef failed for %s, size %d",
 		  pname, di[id].name, di[id].size);
-	    ncclose(cdfid); return;
+	    ncclose(cdfid); return ++nerrs;
 	}
 	add_dim(&test, &di[id]);	/* keep in-memory netcdf in sync */
     }
@@ -172,21 +172,21 @@ test_ncvardef(path)
 				  va[iv].ndims, va[iv].dims)) == -1) {
 	    error("%s: ncvardef failed", pname);
 	    errvar(&test,&va[iv]); /* prints details about variable */
-	    ncclose(cdfid); return;
+	    ncclose(cdfid); return ++nerrs;
 	}
 	add_var(&test, &va[iv]); /* keep in-memory netcdf in sync */
 	/* check that var id returned is one more than previous var id */
 	if (va_id[iv] != test.nvars - 1) {
 	    error("%s: ncvardef returned %d for var id, expected %d",
 		  pname, va_id[iv], test.nvars-1);
-	    ncclose(cdfid); return;
+	    ncclose(cdfid); return ++nerrs;
 	}
 	/* use ncvarinq to get values just set and compare values */
 	if (ncvarinq(cdfid, va_id[iv], tmp.name, &tmp.type,
 		      &tmp.ndims, tmp.dims, &tmp.natts) == -1) {
 	    error("%s: ncvarinq failed", pname);
 	    errvar(&test,&va[iv]); /* prints details about variable */
-	    ncclose(cdfid); return;
+	    ncclose(cdfid); return ++nerrs;
 	}
 	if (strcmp(tmp.name, va[iv].name) != 0 ||
 	    tmp.type != va[iv].type ||
@@ -212,17 +212,17 @@ test_ncvardef(path)
     if (ncvardef(cdfid, va[0].name, va[0].type,
 		  va[0].ndims, va[0].dims) != -1) {
 	error("%s: ncvardef should not allow redefinition", pname);
-	ncclose(cdfid); return;
+	ncclose(cdfid); return ++nerrs;
     }
     /* try bad type, should fail */
     if (ncvardef(cdfid, "badtype", BAD_TYPE, va[0].ndims, va[0].dims) != -1) {
 	error("%s: ncvardef should have failed on bad type", pname);
-	ncclose(cdfid); return;
+	ncclose(cdfid); return ++nerrs;
     }
     /* try bad ndims, should fail */
     if (ncvardef(cdfid, "badndims", va[0].type, -1, va[0].dims) != -1) {
 	error("%s: ncvardef should have failed on bad ndims", pname);
-	ncclose(cdfid); return;
+	ncclose(cdfid); return ++nerrs;
     }
     /* try bad ids in dims vector, should fail */
     id = va[0].dims[0];
@@ -231,11 +231,11 @@ test_ncvardef(path)
 	!= -1) {
 	error("%s: ncvardef should have failed on negative dim id in dims",
 	      pname);
-	ncclose(cdfid); return;
+	ncclose(cdfid); return ++nerrs;
     }
     if (ncendef (cdfid) == -1) {
 	error("%s: ncendef failed", pname);
-	ncclose(cdfid); return;
+	ncclose(cdfid); return ++nerrs;
     }
 
     /* try reading a value of each type, should get appropriate fill value */
@@ -335,7 +335,7 @@ test_ncvardef(path)
 
     if (ncclose (cdfid) == -1) {
 	error("%s: ncclose failed", pname);
-	return;
+	return ++nerrs;
     }
     free (tmp.dims);
     free (tmp.name);
@@ -346,4 +346,6 @@ test_ncvardef(path)
       (void) fprintf(stderr,"FAILED! ***\n");
     else
       (void) fprintf(stderr,"ok ***\n");
+
+    return nerrs;
 }
